pytest_plugins = ["pytester"]


def test_plugin(testdir):
    testdir.makeconftest(
        """
        import pytest

        from hedwig.models import Message, MessageType
        from hedwig.testing.factories import MessageFactory


        @pytest.fixture(name='message_data')
        def _message_data():
            return MessageFactory.build(msg_type=MessageType.trip_created)


        @pytest.fixture()
        def message(message_data):
            return Message(message_data)

        @pytest.fixture(name='other_message_data')
        def _other_message_data():
            return MessageFactory.build(msg_type=MessageType.trip_created)
    """
    )

    # create a temporary pytest test file
    testdir.makepyfile(
        """
        from hedwig.models import MessageType


        def test_mock_hedwig_publish_no_publish(mock_hedwig_publish):
            mock_hedwig_publish.assert_message_not_published(MessageType.trip_created)


        def test_mock_hedwig_publish_publish_check(mock_hedwig_publish, message):
            message.publish()
            mock_hedwig_publish.assert_message_not_published(MessageType.device_created)


        def test_mock_hedwig_publish_publish_check_same_type(mock_hedwig_publish, message, other_message_data):
            message.publish()
            mock_hedwig_publish.assert_message_not_published(message.type, data=other_message_data)
            mock_hedwig_publish.assert_message_published(message.type, data=message.data, version=message.data_schema_version)


        def test_mock_hedwig_publish_published(mock_hedwig_publish, message):
            message.publish()
            mock_hedwig_publish.assert_message_published(message.type, data=message.data, version=message.data_schema_version)
    """
    )

    # run all tests with pytest
    result = testdir.runpytest()

    # check that all 3 tests passed
    result.assert_outcomes(passed=4)



import pprint
from distutils.version import StrictVersion
from typing import Optional, Union, Generator
from unittest import mock

import pytest


__all__ = ['mock_hedwig_publish']


class HedwigPublishMock(mock.MagicMock):
    """
    Custom mock class used by :meth:`hedwig.testing.pytest_plugin.mock_hedwig_publish` to mock the publisher.
    """

    def _message_published(self, msg_type, data: Optional[dict], version: Union[str, StrictVersion]) -> bool:
        return any(
            msg.type == msg_type and (data is None or msg.data) == data and msg.data_schema_version == version
            for (msg,), _ in self.call_args_list
        )

    def _error_message(self) -> str:
        return pprint.pformat([(msg.type, msg.data, msg.data_schema_version) for (msg,), _ in self.call_args_list])

    def assert_message_published(
        self, msg_type, data: Optional[dict] = None, version: Union[str, StrictVersion] = StrictVersion('1.0')
    ) -> None:
        """
        Helper function to check if a Hedwig message with given type, data
        and schema version was sent.
        """
        if not isinstance(version, StrictVersion):
            version = StrictVersion(version)

        assert self._message_published(msg_type, data, version), self._error_message()

    def assert_message_not_published(
        self, msg_type, data: Optional[dict] = None, version: Union[str, StrictVersion] = StrictVersion('1.0')
    ) -> None:
        """
        Helper function to check that a Hedwig message of given type, data
        and schema was NOT sent.
        """
        if not isinstance(version, StrictVersion):
            version = StrictVersion(version)

        assert not self._message_published(msg_type, data, version), self._error_message()


@pytest.fixture()
def mock_hedwig_publish() -> Generator[HedwigPublishMock, None, None]:
    """
    A pytest fixture that mocks Hedwig publisher and lets you verify that your test publishes appropriate messages.
    """
    from hedwig.publisher import publish

    with mock.patch(
        'hedwig.publisher.publish', wraps=publish, new_callable=HedwigPublishMock
    ) as mock_publish, mock.patch('hedwig.publisher._publish_over_sns'):
        yield mock_publish



import logging

import pytest

import hedwig.conf
from hedwig import models
from hedwig.models import Message, MessageType
from hedwig.testing.factories import MessageFactory


def pytest_configure():
    logging.basicConfig()


@pytest.fixture
def settings():
    """
    Use this fixture to override settings. Changes are automatically reverted
    """
    overrides = {}
    original_module = hedwig.conf.settings._user_settings

    class Wrapped:
        def __getattr__(self, name):
            return overrides.get(name, getattr(original_module, name))

    hedwig.conf.settings._user_settings = Wrapped()
    hedwig.conf.settings.clear_cache()
    # in case a test overrides HEDWIG_DATA_VALIDATOR_CLASS
    models._validator = None

    try:
        yield hedwig.conf.settings._user_settings
    finally:
        hedwig.conf.settings._user_settings = original_module
        hedwig.conf.settings.clear_cache()
        # in case a test overrides HEDWIG_DATA_VALIDATOR_CLASS
        models._validator = None


@pytest.fixture(name='message_data')
def _message_data():
    return MessageFactory.build(msg_type=MessageType.trip_created)


@pytest.fixture()
def message(message_data):
    return Message(message_data)



import json
from unittest import mock

import pytest

from hedwig.commands import PartialFailure, requeue_dead_letter
from hedwig.consumer import get_default_queue_name


@mock.patch('hedwig.commands.get_queue_messages', autospec=True)
@mock.patch('hedwig.commands.get_queue', autospec=True)
def test_requeue_dead_letter(mock_get_queue, mock_get_queue_messages):
    num_messages = 3
    visibility_timeout = 4

    messages = [mock.MagicMock() for _ in range(num_messages)]
    mock_get_queue_messages.side_effect = iter([messages, None])
    dlq_name = f'{get_default_queue_name()}-DLQ'

    mock_queue, mock_dlq = mock.MagicMock(), mock.MagicMock()
    mock_queue.attributes = {'RedrivePolicy': json.dumps({'deadLetterTargetArn': dlq_name})}
    mock_queue.send_messages.return_value = {'Failed': []}
    mock_get_queue.side_effect = iter([mock_queue, mock_dlq])
    mock_dlq.delete_messages.return_value = {'Failed': []}

    requeue_dead_letter(num_messages, visibility_timeout)

    mock_get_queue.assert_has_calls([mock.call(get_default_queue_name()), mock.call(dlq_name)])

    mock_get_queue_messages.assert_has_calls(
        [
            mock.call(mock_dlq, num_messages=num_messages, visibility_timeout=visibility_timeout, wait_timeout_s=1),
            mock.call(mock_dlq, num_messages=num_messages, visibility_timeout=visibility_timeout, wait_timeout_s=1),
        ]
    )

    mock_queue.send_messages.assert_called_once_with(
        Entries=[
            {
                'Id': queue_message.message_id,
                'MessageBody': queue_message.body,
                'MessageAttributes': queue_message.message_attributes,
            }
            for queue_message in messages
        ]
    )

    mock_dlq.delete_messages.assert_called_once_with(
        Entries=[
            {'Id': queue_message.message_id, 'ReceiptHandle': queue_message.receipt_handle}
            for queue_message in messages
        ]
    )


@mock.patch('hedwig.commands.get_queue_messages', autospec=True)
@mock.patch('hedwig.commands.get_queue', autospec=True)
def test_requeue_dead_letter_failure(mock_get_queue, mock_get_queue_messages):
    num_messages = 3
    visibility_timeout = 4

    messages = [mock.MagicMock() for _ in range(num_messages)]
    mock_get_queue_messages.side_effect = iter([messages, None])
    dlq_name = f'{get_default_queue_name()}-DLQ'

    mock_queue, mock_dlq = mock.MagicMock(), mock.MagicMock()
    mock_queue.attributes = {'RedrivePolicy': json.dumps({'deadLetterTargetArn': dlq_name})}
    mock_queue.send_messages.return_value = {'Failed': [{'Id': 'string'}], 'Successful': []}
    mock_get_queue.side_effect = iter([mock_queue, mock_dlq])

    with pytest.raises(PartialFailure) as exc_info:
        requeue_dead_letter(num_messages, visibility_timeout)

    assert exc_info.value.success_count == 0
    assert exc_info.value.failure_count == 1

    mock_get_queue.assert_has_calls([mock.call(get_default_queue_name()), mock.call(dlq_name)])

    # not called a 2nd time after failure
    mock_get_queue_messages.assert_called_once_with(
        mock_dlq, num_messages=num_messages, visibility_timeout=visibility_timeout, wait_timeout_s=1
    )

    mock_queue.send_messages.assert_called_once_with(
        Entries=[
            {
                'Id': queue_message.message_id,
                'MessageBody': queue_message.body,
                'MessageAttributes': queue_message.message_attributes,
            }
            for queue_message in messages
        ]
    )

    mock_dlq.delete_messages.assert_not_called()



import json
import logging
import copy
import typing
from decimal import Decimal
from unittest import mock

import boto3
from botocore.config import Config
from retrying import retry

from hedwig.conf import settings
from hedwig.models import Message


log = logging.getLogger(__name__)


def _get_sns_client():
    # https://botocore.readthedocs.io/en/stable/reference/config.html
    # seconds
    config = Config(connect_timeout=settings.AWS_CONNECT_TIMEOUT_S, read_timeout=settings.AWS_READ_TIMEOUT_S)

    return boto3.client(
        'sns',
        region_name=settings.AWS_REGION,
        aws_access_key_id=settings.AWS_ACCESS_KEY,
        aws_secret_access_key=settings.AWS_SECRET_KEY,
        aws_session_token=settings.AWS_SESSION_TOKEN,
        endpoint_url=settings.AWS_ENDPOINT_SNS,
        config=config,
    )


def _get_sns_topic(message: Message) -> str:
    return f'arn:aws:sns:{settings.AWS_REGION}:{settings.AWS_ACCOUNT_ID}:hedwig-{message.topic}'


@retry(stop_max_attempt_number=3, stop_max_delay=3000)
def _publish_over_sns(topic: str, message_json: str, message_attributes: dict) -> dict:
    # transform (http://boto.cloudhackers.com/en/latest/ref/sns.html#boto.sns.SNSConnection.publish)
    message_attributes = {k: {'DataType': 'String', 'StringValue': str(v)} for k, v in message_attributes.items()}
    client = _get_sns_client()
    response = client.publish(TopicArn=topic, Message=message_json, MessageAttributes=message_attributes)
    return response


def _log_published_message(message_body: dict, message_id: str) -> None:
    log.debug('Sent message', extra={'message_body': message_body, 'message_id': message_id})


def _decimal_json_default(obj: Decimal) -> typing.Union[int, float]:
    if isinstance(obj, Decimal):
        int_val = int(obj)
        if int_val == obj:
            return int_val
        else:
            return float(obj)
    raise TypeError


def _convert_to_json(data: dict) -> str:
    return json.dumps(data, default=_decimal_json_default)


def dispatch_mock_sqs_message(message: Message) -> None:
    from hedwig import consumer

    sqs_message = mock.Mock()
    sqs_message.body = json.dumps(message.as_dict())
    sqs_message.receipt_handle = 'test-receipt'
    settings.HEDWIG_PRE_PROCESS_HOOK(sqs_queue_message=sqs_message)
    consumer.message_handler_sqs(sqs_message)


def publish(message: Message) -> None:
    """
    Publishes a message on Hedwig topic
    """
    if settings.HEDWIG_SYNC:
        dispatch_mock_sqs_message(message)
        return

    message_body = message.as_dict()

    headers = {**settings.HEDWIG_DEFAULT_HEADERS(message=message), **message_body['metadata']['headers']}
    # make a copy to prevent changing "headers" variable contents in
    # pre serialize hook
    message_body['metadata']['headers'] = copy.deepcopy(headers)
    settings.HEDWIG_PRE_SERIALIZE_HOOK(message_data=message_body)
    payload = _convert_to_json(message_body)

    topic = _get_sns_topic(message)
    response = _publish_over_sns(topic, payload, headers)

    _log_published_message(message_body, response['MessageId'])



import typing


class RetryException(Exception):
    """
    Special exception that does not log an exception when it is received.
    This is a retryable error.
    """

    def __init__(self, *args, **kwargs) -> None:
        pass


class LoggingException(Exception):
    """
    An exception that allows passing additional logging info. `extra` must be a dict that will be passed to
    `logging.exception` and can be used by a logging adaptor etc.
    """

    def __init__(self, message, extra: typing.Optional[typing.Dict] = None) -> None:
        super().__init__(message)
        self.extra = extra


class IgnoreException(Exception):
    """
    Indicates that this task should be ignored.
    """

    pass


class ValidationError(Exception):
    """
    Message failed JSON schema validation
    """

    pass


class ConfigurationError(Exception):
    """
    There was some problem with settings
    """

    pass


class CallbackNotFound(Exception):
    """
    No callback found that can handle the given message. Check your `CALLBACKS` settings.
    """

    pass



import time
import typing
import uuid

import factory
from factory import fuzzy

from hedwig.models import Message, MessageType
from hedwig.conf import settings


class HeadersFactory(factory.DictFactory):
    request_id = factory.Faker('uuid4')


class MetadataFactory(factory.DictFactory):
    timestamp = factory.LazyFunction(lambda: int(time.time() * 1000))
    publisher = settings.HEDWIG_PUBLISHER
    headers = factory.SubFactory(HeadersFactory)


class DataFactory(factory.DictFactory):
    trip_id = 'T_1234567890123456'
    user_id = 'U_1234567890123456'
    device_id = 'abcdef00abcdef00abcdef00'
    vin = '00000000000000000'
    vehicle_id = 'C_1234567890123456'


_SCHEMA_FORMAT = 'https://hedwig.automatic.com/schema#/schemas/{}/{}.{}'


class MessageFactory(factory.DictFactory):
    class Meta:
        model = Message

    class Params:
        model_version = 1
        addition_version = 0
        msg_type = fuzzy.FuzzyChoice(list(MessageType))

    format_version = str(Message.FORMAT_CURRENT_VERSION)
    schema = factory.LazyAttribute(
        lambda obj: _SCHEMA_FORMAT.format(obj.msg_type.value, obj.model_version, obj.addition_version)
    )
    id = factory.LazyFunction(lambda: str(uuid.uuid4()))
    metadata = factory.SubFactory(MetadataFactory)
    data = factory.SubFactory(DataFactory)

    @classmethod
    def _build(cls, model_class: typing.Type[Message], *args, **kwargs) -> typing.Dict:
        if args:
            raise ValueError("MessageFactory %r does not support Meta.inline_args.", cls)

        return kwargs

    @classmethod
    def _create(cls, model_class: typing.Type[Message], *args, **kwargs) -> Message:
        return model_class(cls._build(model_class, *args, **kwargs))



#!/usr/bin/env python

import os
import importlib
import typing

try:
    from django.conf import settings as django_settings
    from django.dispatch import receiver
    from django.test import signals

    HAVE_DJANGO = True
except ImportError:
    HAVE_DJANGO = False


_DEFAULTS = {
    'AWS_REGION': None,
    'AWS_ACCOUNT_ID': None,
    'AWS_ACCESS_KEY': None,
    'AWS_CONNECT_TIMEOUT_S': 2,
    'AWS_ENDPOINT_SNS': None,
    'AWS_ENDPOINT_SQS': None,
    'AWS_READ_TIMEOUT_S': 2,
    'AWS_SECRET_KEY': None,
    'AWS_SESSION_TOKEN': None,
    'HEDWIG_CALLBACKS': {},
    'HEDWIG_DATA_VALIDATOR_CLASS': 'hedwig.validator.MessageValidator',
    'HEDWIG_DEFAULT_HEADERS': 'hedwig.conf.default_headers_hook',
    'HEDWIG_MESSAGE_ROUTING': {},
    'HEDWIG_POST_DESERIALIZE_HOOK': 'hedwig.conf.noop_hook',
    'HEDWIG_PRE_PROCESS_HOOK': 'hedwig.conf.noop_hook',
    'HEDWIG_PRE_SERIALIZE_HOOK': 'hedwig.conf.noop_hook',
    'HEDWIG_PUBLISHER': None,
    'HEDWIG_QUEUE': None,
    'HEDWIG_SCHEMA_FILE': None,
    'HEDWIG_SYNC': False,
}


# List of settings that may be in string import notation.
_IMPORT_STRINGS = (
    'HEDWIG_DATA_VALIDATOR_CLASS',
    'HEDWIG_DEFAULT_HEADERS',
    'HEDWIG_POST_DESERIALIZE_HOOK',
    'HEDWIG_PRE_PROCESS_HOOK',
    'HEDWIG_PRE_SERIALIZE_HOOK',
)

# List of settings that will be dicts with values as string import notation.
_IMPORT_DICT_VALUES = ('HEDWIG_CALLBACKS',)


def default_headers_hook(*args, **kwargs) -> typing.Dict[str, str]:
    return {}


def noop_hook(*args, **kwargs) -> None:
    pass


class _LazySettings:
    """
    A settings object, that allows settings to be accessed as properties.
    For example:
        from hedwig.conf import settings
        print(settings.AWS_REGION)
    Any setting with string import paths will be automatically resolved
    and return the class, rather than the string literal.
    """

    def __init__(self) -> None:
        self._defaults = _DEFAULTS
        self._import_strings = _IMPORT_STRINGS
        self._import_dict_values = _IMPORT_DICT_VALUES
        self._user_settings = None

        if os.environ.get("SETTINGS_MODULE"):
            self._user_settings = importlib.import_module(os.environ["SETTINGS_MODULE"], package=None)
        elif HAVE_DJANGO:
            # automatically import Django settings in Django projects
            self._user_settings = django_settings
        else:
            raise ImportError("No settings module found to import")

    @staticmethod
    def _import_string(
        dotted_path_or_callable: typing.Union[typing.Callable, str]
    ) -> typing.Union[typing.Callable, typing.Type]:
        """
        Import a dotted module path and return the attribute/class designated by the
        last name in the path. Raise ImportError if the import failed.
        """
        if callable(dotted_path_or_callable):
            return dotted_path_or_callable

        try:
            module_path, class_name = dotted_path_or_callable.rsplit('.', 1)
        except ValueError as err:
            raise ImportError(f"{dotted_path_or_callable} doesn't look like a module path") from err

        module = importlib.import_module(module_path)

        try:
            return getattr(module, class_name)
        except AttributeError as err:
            raise ImportError(f"Module '{module_path}' does not define a '{class_name}' attribute/class") from err

    def __getattr__(self, attr: str) -> typing.Any:
        if attr not in self._defaults:
            raise AttributeError("Invalid API setting: '%s'" % attr)

        try:
            # Check if present in user settings
            val = getattr(self._user_settings, attr)
        except AttributeError:
            # Fall back to defaults
            val = self._defaults[attr]

        # Coerce import strings into classes
        if attr in self._import_strings:
            val = self._import_string(val)

        if attr in self._import_dict_values:
            val = {k: self._import_string(v) for k, v in val.items()}

        # Cache the result
        setattr(self, attr, val)
        return val

    def clear_cache(self) -> None:
        for attr in self._defaults:
            try:
                delattr(self, attr)
            except AttributeError:
                pass


if HAVE_DJANGO:

    @receiver(signals.setting_changed)
    def clear_cache_on_setting_override(sender, setting, value, enter, **kwargs):
        settings.clear_cache()


settings = _LazySettings()



import os


# to test callables in HEDWIG_CALLBACKS
def device_handler(message):
    pass


AWS_ACCESS_KEY = "DUMMY_KEY"
AWS_ACCOUNT_ID = "DUMMY_ACCOUNT"
AWS_REGION = "DUMMY_REGION"
AWS_SECRET_KEY = "DUMMY_SECRET"

AWS_CONNECT_TIMEOUT_S = 5
AWS_READ_TIMEOUT_S = 5

HEDWIG_QUEUE = 'DEV-MYAPP'

HEDWIG_SCHEMA_FILE = os.path.abspath('tests/schema.json')

HEDWIG_CALLBACKS = {
    ('trip_created', '1.*'): 'tests.handlers.trip_created_handler',
    ('trip_created', '2.*'): 'tests.handlers.trip_created_handler',
    ('device.created', '1.*'): device_handler,
}

HEDWIG_MESSAGE_ROUTING = {
    ('trip_created', '1.*'): 'dev-trip-created',
    ('trip_created', '2.*'): 'dev-trip-created',
    ('device.created', '1.*'): 'dev-device-created',
    ('vehicle_created', '1.*'): 'dev-vehicle-created',
}

HEDWIG_PUBLISHER = 'myapp'



import json
import logging

import funcy

from hedwig.consumer import get_default_queue_name, get_queue, get_queue_messages


class PartialFailure(Exception):
    """
    Error indicating either send_messages or delete_messages API call failed partially
    """

    def __init__(self, result):
        self.success_count = len(result['Successful'])
        self.failure_count = len(result['Failed'])
        self.result = result


def _enqueue_messages(queue, queue_messages) -> None:
    params: dict = {}

    result = queue.send_messages(
        Entries=[
            funcy.merge(
                {'Id': queue_message.message_id, 'MessageBody': queue_message.body},
                {'MessageAttributes': queue_message.message_attributes} if queue_message.message_attributes else {},
                params,
            )
            for queue_message in queue_messages
        ]
    )
    if result.get('Failed'):
        raise PartialFailure(result)


def get_dead_letter_queue(queue):
    queue_name = json.loads(queue.attributes['RedrivePolicy'])['deadLetterTargetArn'].split(':')[-1]
    return get_queue(queue_name)


def requeue_dead_letter(num_messages: int = 10, visibility_timeout: int = None) -> None:
    """
    Re-queues everything in the Hedwig DLQ back into the Hedwig queue.

    :param num_messages: Maximum number of messages to fetch in one SQS call. Defaults to 10.
    :param visibility_timeout: The number of seconds the message should remain invisible to other queue readers.
        Defaults to None, which is queue default
    """
    queue = get_queue(get_default_queue_name())

    dead_letter_queue = get_dead_letter_queue(queue)

    logging.info("Re-queueing messages from {} to {}".format(dead_letter_queue.url, queue.url))
    while True:
        queue_messages = get_queue_messages(
            dead_letter_queue, num_messages=num_messages, visibility_timeout=visibility_timeout, wait_timeout_s=1
        )
        if not queue_messages:
            break

        logging.info("got {} messages from dlq".format(len(queue_messages)))

        _enqueue_messages(queue, queue_messages)
        dead_letter_queue.delete_messages(
            Entries=[{'Id': message.message_id, 'ReceiptHandle': message.receipt_handle} for message in queue_messages]
        )

        logging.info("Re-queued {} messages".format(len(queue_messages)))



import inspect
import typing

from hedwig.exceptions import ConfigurationError, CallbackNotFound
from hedwig.models import Message, MessageType
from hedwig.conf import settings


class Callback:
    def __init__(self, fn: typing.Callable) -> None:
        self._fn = fn
        signature = inspect.signature(fn)
        message_found = False
        for p in signature.parameters.values():
            # if **kwargs is specified, just pass all things by default since function can always inspect arg names
            if p.kind == inspect.Parameter.VAR_KEYWORD:
                # disallow use of *kwargs
                raise ConfigurationError("Use of **kwargs is not allowed")
            elif p.kind == inspect.Parameter.VAR_POSITIONAL:
                # disallow use of *args
                raise ConfigurationError("Use of *args is not allowed")
            elif p.name == 'message':
                if p.annotation is not inspect.Signature.empty and p.annotation is not Message:
                    raise ConfigurationError("Signature for 'message' param must be `hedwig.Message`")
                message_found = True
            else:
                raise ConfigurationError(f"Unknown param '{p.name}' not allowed")

        if not message_found:
            raise ConfigurationError("Callback must accept a parameter called 'message'")

    @property
    def fn(self) -> typing.Callable:
        """"
        return: Task function
        """
        return self._fn

    def call(self, message: Message) -> None:
        """
        Calls the task with this message

        :param message: The message
        """
        self.fn(message)

    def __str__(self) -> str:
        return f'Hedwig task: {self.fn.__name__}'

    @classmethod
    def find_by_message(cls, msg_type: MessageType, major_version: int) -> 'Callback':
        """
        Finds a callback by message type
        :return: Callback
        :raises CallbackNotFound: if task isn't registered
        """
        version_pattern = f'{major_version}.*'
        if (msg_type, version_pattern) in _ALL_CALLBACKS:
            return _ALL_CALLBACKS[(msg_type, version_pattern)]

        raise CallbackNotFound(msg_type, major_version)


_ALL_CALLBACKS = {(MessageType(k[0]), k[1]): Callback(v) for k, v in settings.HEDWIG_CALLBACKS.items()}



"""A setuptools based setup module.
See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

import ast
from codecs import open
from os import path
import re
from setuptools import setup, find_packages


cwd = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(cwd, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

_version_re = re.compile(r'VERSION\s+=\s+(.*)')

with open('hedwig/__init__.py', 'rb') as f:
    version = str(ast.literal_eval(_version_re.search(f.read().decode('utf-8')).group(1)))

tests_require = [
    'pytest',
    'flake8',
    'mypy',
    'pytest-env',
    'ipdb',
    'factory-boy',
    'coverage',
    'coveralls',
    'pytest-cov',
    'black',
]

setup(
    name='authedwig',
    version=version,
    description='Hedwig Python Library',
    long_description=long_description,
    url='https://github.com/Automatic/hedwig-python',
    author='Automatic Labs',
    license='Apache Software License (Apache License 2.0)',
    maintainer='Aniruddha Maru',
    maintainer_email='aniruddhamaru@gmail.com',
    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Natural Language :: English',
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'License :: OSI Approved :: Apache Software License',
    ],
    python_requires='>=3.6',
    keywords='python authedwig hedwig',
    # https://mypy.readthedocs.io/en/latest/installed_packages.html
    package_data={'hedwig': ['py.typed']},
    packages=find_packages(exclude=['contrib', 'docs', 'tests', 'tests.*']),
    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=['funcy', 'retrying', 'boto3', 'jsonpointer', 'jsonschema'],
    tests_require=tests_require,
    setup_requires=['pytest-runner'],
    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    extras_require={
        'dev': ['flake8', 'Sphinx>=1.7.2', 'sphinx-autodoc-typehints'],
        'test': tests_require,
        'publish': ['bumpversion', 'twine'],
    },
    include_package_data=True,
    # the following makes a plugin available to pytest
    entry_points={'pytest11': ['hedwig = hedwig.testing.pytest_plugin']},
)
